function [f,v] = cmfuzz(x,c,m,e,fh)
% CMfuzz Fuzzy c-means clustering algorithm
%
% !! Special version for fuzz exercises !!
%
% [F V] = cmfuzz(X,c,m,e,s)
%     Input:
%       X  ... M by N data matrix, M is the number of data points
%              and N the dimension of X
%       c ... the number of clusters. 
%       m  ... (optional) parameter m > 1, determines the "fuzziness"
%              of clustering, for m close to 1 clusters become crisp
%              default value is 2.
%       e  ... (optional) termination tolerance, the algorithm stops
%              when |F(k-1) - F(k)| <= e, default 1e-3.
%       fh  ... figure handle for plotting
%
%     Output:
%       F  ... fuzzy partition matrix
%       V  ... centroid prototype matrix

% (c) Robert Babuska, June 1993
% Edited for fuzz by Magne 1998

[mx,nx] = size(x);
x1 = ones(mx,1);

if (nargin < 3), m = 2; elseif isempty(m), m = 2; end;
if (nargin < 4), e = 1e-3; elseif isempty(e), e = 1e-3; end;

rand('seed',2054);	% set the random generator seed

% Initialize fuzzy partition matrix
   f0  = zeros(c,mx);
   for j=1:mx
     cc = 1 + fix((j-1)/round(mx/c));
     while (cc > c), cc = cc - 1; end
     f0(cc,j) = 0.6+0.4*rand;
     for i=1:c
       sumf0 = sum(f0(:,j));
       if (i ~= cc)
         f0(i,j) = 1-sumf0;
         if (i ~= c), f0(i,j) = f0(i,j)*rand; end;
       end
       if (c ~= 1 & cc==c & i==c), f0(i-1,j) = f0(i-1,j) + 1 - sumf0; end
     end
   end
   f0 = (f0.*(f0 > 0))';

f = zeros(mx,c);

% initialize graphics
	figure(fh);set(fh,'Name','Fuzzy C-Means Clustering');
	subplot(211);
    v = ones(c,1)*mean(x);
	H1 = plot(x(:,1),x(:,2),'go',v(:,1),v(:,2),'ro','EraseMode','xor');
    minx = min(x(:,1)); maxx = max(x(:,1));
    miny = min(x(:,2)); maxy = max(x(:,2));
    axis([minx maxx miny maxy]);set(gca,'box','off');
    set(H1(2),'LineWidth',3);
	subplot(212);
	H2 = plot(x(:,1),f0,'.','markersize',12,'EraseMode','xor');
	title('Membership functions');
	xlabel('x'); ylabel('Membership');
    set(gcf,'UserData',[H1;H2]);


% Iterate
iter = 0;
while  max(max( (f0-f) > e ))
  iter = iter + 1;
  f = f0;
  fm = f.^m; sumf = sum(fm);
% Calculate centers
  v = (fm'*x)./(sumf'*ones(1,nx));
% Calculate distances
    for j = 1 : c
      xv = x - x1*v(j,:);
      d(:,j) = sum([0*xv (xv.^2)]')';
    end;
  J(iter) = sum(sum(f0.*d));
% Update f0
  d = d.^(1/(m-1));
  sumd = sum((d.^(-1))')'*ones(1,c);
  f0 = (d .* sumd).^(-1);

% Plot intermediate results
    H = get(gcf,'UserData');
    set(H(2),'xdata',v(:,1),'ydata',v(:,2));
	for i = 1 : c,
		set(H(2+i),'ydata',f0(:,i));
    end;
	drawnow;
end
f = f0;
