function [f,v,P,V] = gkfuzz(x,c,m,e,fh)
% Gustastafson-Kessel clustering algorithm (with fuzzy covariance matrix).
%
% !! Special version for fuzz exercises !!
% 
% [F,C,P,V] = GKfuzz(X,c,m,e,s,fh)
%     Input:
%       X  ... M by N data matrix,
%              M is the number of data points and N data dimension
%       c ... the number of clusters.
%
%       m  ... optional parameter m > 1, determines the fuzziness
%              of clustering, for m close to 1 clusters become crisp,
%              default value is 2
%       e  ... optional termination tolerance, the algorithm stops
%              when max(max(F(k-1) - F(k))) <= e, default tolerance 1e-3
%       fh  ... figure handle for plotting
%
%     Output:
%       F  ... fuzzy partition matrix
%       C  ... cluster means matrix
%       P  ... cluster covariance matrices concatenated in one matrix
%              P = [P1;P2;...Pk], where k is number of clusters
%       V  ... eigenvectors of the covariance matrices, corresponding
%              to the smallest eigenvalues

% (c) Robert Babuska, 1994-96
% edited for fuzz by Magne Setnes, 1998

if (nargin < 3), m = 2;    elseif isempty(m),   m = 2; end;
if (nargin < 4), e = 1e-2; elseif isempty(e),   e = 1e-2; end;

[mx,nx] = size(x);
x1 = ones(mx,1);
inx = 1/nx;
Jmin = inf;

% Initialize fuzzy partition matrix
rand('seed',2054);	% set the random generator seed
  mm = mean(x);
  aa = max(abs(x - ones(mx,1)*mm));
  v = 2*(ones(c,1)*aa).*(rand(c,nx)-0.5) + ones(c,1)*mm;
  for j = 1 : c,       
    xv = x - x1*v(j,:);
    d(:,j) = sum((xv.^2)')';
  end;
  [mm,ind] = min(d);
  v = x(ind,:);

% Calculate f0
  for j = 1 : c,                        % for all clusters
    xv = x - x1*v(j,:);
    d(:,j) = sum((xv.^2)')';
  end;
  d = (d+1e-10).^(-1/(m-1));
  f0 = (d ./ (sum(d')'*ones(1,c)));

f = zeros(mx,c);                % partition matrix
iter = 0;                       % iteration counter
rho = ones(1,c);

% initialize graphics
	figure(fh);set(fh,'Name','Fuzzy GK Clustering');
    subplot(211);
    lines = [v(:,1)*x(:,1)'+v(:,2)*ones(1,mx)]';
    mask = find(f0 < 0.2);                   % find membership degrees < 0.2
    lines(mask) = NaN*ones(size(mask));      % mask with NaN's for plots
    H1 = plot(x(:,1),x(:,2),'go',v(:,1),v(:,2),'r*',x(:,1),lines,'EraseMode','xor');
    title('Function approximation');
    xlabel('x'); ylabel('y');
    minx = min(x(:,1)); maxx = max(x(:,1));
    miny = min(x(:,2)); maxy = max(x(:,2));
    ma = 0.3*max(abs(x(:,2)));
	axis([minx maxx miny-ma maxy+ma]);
    subplot(212);
    H2 = plot(x(:,1),f0,'.','markersize',12,'EraseMode','xor');
    title('Membership functions');
    xlabel('x'); ylabel('Membership');
    set(gcf,'UserData',[H1;H2]);

% Iterate
while  max(max(f0-f)) > e
  iter = iter + 1;
  f = f0;
  fm = f.^m; sumf = sum(fm);
% Calculate centers
  v = (fm'*x)./(sumf'*ones(1,nx));
  for j = 1 : c,                        % for all clusters
    xv = x - x1*v(j,:);
% Calculate covariance matrix
    p = ones(nx,1)*fm(:,j)'.*xv'*xv/sumf(j);
% Calculate distances
    M = (det(p)/rho(j))^inx*inv(p);
    d(:,j) = sum((xv*M.*xv)')';
% Calculate eigen vectors and cluster prototypes (lines)
      [ev,ed] = eig(p); ed = diag(ed);
      ev = ev(:,ed == min(ed));
      lines(:,j) = -x(:,1)*ev(1)/ev(2) + v(j,:)*ev/ev(2);
      mask = find(f0 < 0.2);                   % find membership degrees < 0.2
      lines(mask) = NaN*ones(size(mask));      % mask with NaN's for plots
end;

J = sum(sum(f0.*d));
% Update f0
  d = (d+1e-10).^(-1/(m-1));
  f0 = (d ./ (sum(d')'*ones(1,c)));

% Plot intermediate results
      H = get(gcf,'UserData');
      set(H(2),'xdata',v(:,1),'ydata',v(:,2));
      for i = 1 : c,
        set(H(2+i),'ydata',lines(:,i));
        set(H(2+c+i),'ydata',f0(:,i));
      end;
      drawnow;
end


if J < Jmin, Jmin = J; 
    f1=f0; v1=v; 
end;

f = f1; fm = f.^m; sumf = sum(fm); v=v1;

P = zeros(c*nx,nx);             % covariance matrix
V = zeros(c,nx);                % eigenvectors

% calculate P,V,D,M
for j = 1 : c,                        % for all clusters
    xv = x - ones(mx,1)*v(j,:);
% Calculate covariance matrix
    p = ones(nx,1)*fm(:,j)'.*xv'*xv/sumf(j);
% Calculate eigen values and eigen vectors
    [ev,ed] = eig(p); ed = diag(ed)';
    ev = ev(:,ed == min(ed));
% Concatenate cluster info in one matrix
    P((j-1)*nx+(1:nx),:) = p;
    V(j,:) = ev';
end;
return
